"""
Este archivo incluye un ejemplo de código con objetos

       Enero de 2022

    Angel Luis Aldana Valverde
        angel.l.aldana@prohimet.org
        https://www.edaphi.es
        https://www.angel-l-aldana.com



Cómo funciona:
Se generan bichos que se sitúan en un tablero. En un cierto porcentaje, algunos bichos son depredadores. En cada
iteracción, se intenta mover cada bicho, si es depredador y la posición está ocupada por un no depredador, este
último es eliminado.

Descubre cómo funciona, completa la documentación del código incluyendo comentarios, y experimenta cambiando
las reglas del juego.

"""

import random


class Bicho:
    """
    Cada instancia de bicho se reconocerá por su propiedad ind, que debe ser única en el tablero
    """
    def __init__(self):
        self.ind = 0
        self.fila = 0
        self.columna = 0
        self.depredador = True

    def intenta_mover(self):
        """
        Propone un movimiento desde sus coordenadas actuales
        :return:
        """
        def aleatorio(c: int):
            r = random.random()
            if r < 0.45:
                res = c + 1
            elif r > 0.55:
                res = c - 1
            else:
                res = c
            return res

        return aleatorio(self.fila), aleatorio(self.columna)


class Tablero:
    """
    Tablero en el que se sitúan y mueven los bichos
    """
    def __init__(self):
        self.porcentaje_d = 10
        self.posiciones_fc = []
        self.bichos = []

    def crea_tablero(self, filas: int, columnas: int):
        """
        Crea el tablero en los que situarán los bichos
        :param filas:
        :param columnas:
        :return:
        """
        self.posiciones_fc = []
        for i in range(0, filas):
            f = [-1 for i in range(0, columnas)]
            self.posiciones_fc.append(f)

    def filas(self):
        return len(self.posiciones_fc)

    def columnas(self):
        return len(self.posiciones_fc[0])

    def crea_bichos(self, n):
        """
        Crea bichos columna los sitúa en el tablero
        Requiere la creación previa del tablero
        :param n: número de bichos
        :return:
        """
        if n > self.filas() * self.columnas():
            print("No se pueden crear tantos bichos ({0:d})".format(n))
            print("Número de posiciones_fc posibles: {0:d}".format(self.filas() * self.columnas()))
            exit(0)
        nd = max(1, int(n * self.porcentaje_d / 100))
        self.bichos = []
        for ib in range(n):
            b = Bicho()
            b.ind = ib
            b.depredador = ib <= nd
            i = 0
            j = 0
            while self.posiciones_fc[j][i] != -1:
                print("Buscando posición para {0:d}".format(ib))
                i = random.randint(0, self.columnas() - 1)
                j = random.randint(0, self.filas() - 1)
            b.fila = j
            b.columna = i
            self.posiciones_fc[j][i] = ib
            self.bichos.append(b)
        print("")

    def i_bicho(self, ind):
        ires = -1
        for k in range(0, len(self.bichos)):
            if self.bichos[k].ind == ind:
                ires = k
                break
        return ires

    def bicho(self, i) -> Bicho:
        return self.bichos[self.i_bicho(ind=i)]

    def n_depredadores(self):
        r = 0
        for i in range(0, len(self.bichos)):
            if self.bicho(i).depredador:
                r += 1
        return r

    def mueve_bichos(self):
        n_bichos_ini = len(self.bichos)
        for i in range(0, n_bichos_ini):
            if self.i_bicho(ind=i) >= 0:
                b = self.bicho(i)
                y, x = b.intenta_mover()
                y = max(0, min(y, self.filas() - 1))
                x = max(0, min(x, self.columnas() - 1))
                if self.posiciones_fc[y][x] == -1:
                    self.posiciones_fc[b.fila][b.columna] = -1  # Se libera
                    b.fila = y
                    b.columna = x
                    self.posiciones_fc[y][x] = b.ind  # Se ocupa
                else:
                    if self.bicho(i).depredador:
                        if not self.bicho(self.posiciones_fc[y][x]).depredador:
                            print(" {0:d} come a {1:d}".format(b.ind, self.posiciones_fc[y][x]))
                            self.bichos.remove(self.bicho(self.posiciones_fc[y][x]))
                            self.posiciones_fc[b.fila][b.columna] = -1
                            b.fila = y
                            b.columna = x
                            self.posiciones_fc[y][x] = b.ind
                        else:
                            # puedes poner otra cosa
                            pass

    def imprime_tablero(self):
        for j, f in enumerate(self.posiciones_fc):
            for c in range(0, len(f)):
                print('|', end='')
                if f[c] == -1:
                    print("-".center(7, ' '), end='')
                else:
                    if self.bicho(f[c]).depredador:
                        print('{0:d}(D)'.format(f[c]).center(7, ' '), end='')
                    else:
                        print('{0:d}'.format(f[c]).center(7, ' '), end='')
            print("|")
        print('Bichos: {0:d}, depredadores (D): {1:d}'.format(len(self.bichos), self.n_depredadores()))
        print("\n")


def prueba_alcance(b: Bicho):
    b.fila = 0
    return


def main():
    t = Tablero()
    t.porcentaje_d = 10
    t.crea_tablero(filas=5, columnas=7)
    t.crea_bichos(20)
    # Para probar el alcance con objetos
    t.bicho(0).fila = 3
    print(t.bicho(0).fila)
    prueba_alcance(t.bicho(0))
    print(t.bicho(0).fila)
    # Sigue después de la prueba
    t.imprime_tablero()
    resp = ''
    while resp.upper() != "S":
        t.mueve_bichos()
        t.imprime_tablero()
        resp = input("Indique S para terminar: ")
    if len(t.bichos) == t.n_depredadores():
        print("\n\tEra sólo cuestión de tiempo")


if __name__ == "__main__":
    main()













